/* $Workfile: OtxTone.h $   $Revision: 33 $    $Modtime: 8/06/04 3:02p $  */
#ifndef OTX_TONE_H
#define OTX_TONE_H
/*******************************************************************************

  @doc PublicHwApi OtxTone

  Copyright (C) 1998 - 2003 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxTone | OTX Tone Generator/Detector Logical Device Function Prototypes

  @contents2
  OtxTone.h |

  @index func | OtxTone

  @contents2
  OTX Tone Generator/Detector Functions - OtxToneXXX() | Functions available for Tone Generators / Detectors

  @head3 OtxToneXXX() Function Reference |

  @index func | OtxTone

*******************************************************************************/

#include "OtxType.h"
#include "OtxDef.h"

#include "OtxToneD.h"
#include "OtxDataD.h"


#ifdef __cplusplus
extern "C" {
#endif


/*******************************************************************************

  @func
    Set a constant value (a byte or a word depending on the data format)
    to be sent on a Pipe.

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |

    // create 64 Const Val Generator (32 on DSP#0 and 32 on DSP#1)
    for (i = 0; i < 2; i++) {
      OTX_INT32 j;
      for (j = 0; j < 32; j++) {
        if ((i*32 + j)%16 == 0) continue; // Skip 0 (sync), 16 (signalling), 32 (sync), 48 (signalling)
        // create Const Val Generator logical device
        result = OtxDrvCreateLogicalDevice(
                    g_hPhysDevs.hDsp[i], 
                    OTX_LDEVICE_TONE_CONST_VAL_GENERATOR, 
                    g_hEventQueue, 
                    i*32 + j, 
                    &g_sqConstValGenerator[i*32 + j].hHandle); 
      }
    }  

    // pipe creation and connection, see example on 
    @ex <f OtxDrvConnectPipe> |

    // connect logical devices within the DSP
    for (i = 0; i < 2; i++) {
      for (j = 0; j < 32; j++) {
        if ((i*32 + j)%16 == 0) continue; // Skip 0 (sync), 16 (signalling), 32 (sync), 48 (signalling)

          // transmit direction
          ts = j;
          memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
          sqTimeSlotMask[ts] = 0xFF;
          nCapacity = 64;
           
          result = OtxDrvCreatePipe(
                      nCapacity, 
                      &g_sqConstValGenerator[i*32 + j].hPipe); 

          result = OtxDrvConnectPipe(
                      g_sqConstValGenerator[i*32 + j].hPipe, 
                      OTX_PIPE_INPUT,  
                      g_hPhysDevs.hDsp[i], 
                      -1, 
                      sqTimeSlotMask, 
                      32); 
          result = OtxDrvConnectPipe(
                      g_sqConstValGenerator[i*32 + j].hPipe, 
                      OTX_PIPE_OUTPUT, 
                      g_hPhysDevs.hLi[i],  
                      -1, 
                      sqTimeSlotMask, 
                      32); 
           
          result = OtxDrvConnectLogicalDevice(
                      g_sqConstValGenerator[i*32 + j].hHandle,  
                      -1, 
                      g_sqConstValGenerator[i*32 + j].hPipe,  
                      -1); 


          // initialize the Const Val Generator (hosted in the DSP)
          result = OtxToneConstValGeneratorInit(
                      g_sqConstValGenerator[i*32 + j].hHandle, 
                      OTX_DATA_FORMAT_UINT8, 
                      0x55); 

          // enable Const Val Generator devices
          result = OtxDrvEnable(
                      g_sqConstValGenerator[i].hHandle,  
                      OTX_TASK_SYNC, 
                      5000L); 
      }
    }  

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_CONST_VAL_GENERATOR>
    <nl><t OtxDataFormatE>
    <nl><f OtxToneReadVal>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>
    <nl><f OtxToneDtmfDial>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneConstValGeneratorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Format of the Generated Data
  IN OTX_UINT16     nConstWord        // @parm Constant Value to be sent
);


/*******************************************************************************

  @func
    Configure a sampler device that allows to read a sample of an incoming 
    64kbps pipe.

  @comm
    Call <f OtxToneValSamplerReadData> to read the sample.

  @ex |
    // create 64 Val Sampler (32 on DSP#0 and 32 on DSP#1)
    for (i = 0; i < 2; i++) {
      OTX_INT32 j;
      for (j = 0; j < 32; j++) {
        if ((i*32 + j)%16 == 0) continue; // Skip 0 (sync), 16 (signalling), 32 (sync), 48 (signalling)
        
        // Create Val Sampler logical device
        result = OtxDrvCreateLogicalDevice(
                    g_hPhysDevs.hDsp[i], 
                    OTX_LDEVICE_TONE_VAL_SAMPLER, 
                    g_hEventQueue, 
                    i*32 + j, 
                    &g_sqValSampler[i*32 + j].hHandle); 
      }
    }  

    // pipe creation and connection, see example on 
    @ex <f OtxDrvConnectPipe> |

    // connect logical devices within the DSP
    for (i = 0; i < 2; i++) {
      for (j = 0; j < 32; j++) {
        if ((i*32 + j)%16 == 0) continue; // Skip 0 (sync), 16 (signalling), 32 (sync), 48 (signalling)

          // receive direction:
          ts = j;
          memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
          sqTimeSlotMask[ts] = 0xFF;
          nCapacity = 64;
           
          result = OtxDrvCreatePipe(
                      nCapacity, 
                      &g_sqValSampler[i*32 + j].hPipe); 
          result = OtxDrvConnectPipe(
                      g_sqValSampler[i*32 + j].hPipe, 
                      OTX_PIPE_OUTPUT,  
                      g_hPhysDevs.hDsp[i], 
                      -1, 
                      sqTimeSlotMask, 
                      32); 
          result = OtxDrvConnectPipe(
                      g_sqValSampler[i*32 + j].hPipe, 
                      OTX_PIPE_INPUT,   
                      g_hPhysDevs.hLi[i],  
                      -1, 
                      sqTimeSlotMask, 
                      32); 
           
          result = OtxDrvConnectLogicalDevice(
                      g_sqValSampler[i*32 + j].hPipe, 
                      -1, 
                      g_sqValSampler[i*32 + j].hHandle, 
                      -1); 

          // initialize the Val Sampler (hosted in the DSP)
          result = OtxToneValSamplerInit(
                      g_sqValSampler[i*32 + j].hHandle, 
                      OTX_DATA_FORMAT_UINT8); 

          // enable Const Val Sampler devices
          result = OtxDrvEnable(
                      g_sqValSampler[i*32 + j].hHandle,  
                      OTX_TASK_SYNC, 5000L); 
      }
    }  

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_VAL_SAMPLER>
    <nl><t OtxDataFormatE>
    <nl><f OtxToneValSamplerReadData>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneValSamplerInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat       // @parm Format of the sample 
);


/*******************************************************************************

  @func
    Read an 16-bit sample from a an incoming 64kbps pipe.

  @ex |
      static OTX_INT32 nCurrentDialer = 1;
      OTX_UINT16 nSampleVal;
  
      // get sample value from a Dialer
      result = OtxToneValSamplerReadData(
                  g_sqValSampler[nCurrentDialer].hHandle, 
                  &nSampleVal); 

      // display it
      DemoLogf("0x%.2X \n", nSampleVal);


  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_VAL_SAMPLER>
    <nl><f OtxToneValSamplerInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneValSamplerReadData(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  OUT OTX_UINT16    *pSample          // @parm 8-bit sample from the pipe
);


/*******************************************************************************

  @func
    Set the tone effects to be generated by tone generator data filter

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |
    // create Tone Effect Generators Logical Devices (one per DSP)
    for (nDsp = 0; nDsp < 2; nDsp++) {
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hDsp[nDsp], 
                  OTX_LDEVICE_TONE_EFFECTS_GENERATOR, 
                  g_hEventQueue, 
                  nDsp, 
                  &g_hLogicDevs.hToneGenerator[nDsp]); 
    }

  // connect devices
  {
    OTX_INT32 i;
    OTX_RESULT result;
    OTX_UINT8 sqTimeSlotMask[32];
    OTX_INT32 ts;

    // create Pipes
    for (i = 0; i < 2; i++) {
      // Dsp <--> Codec
      result = OtxDrvCreatePipe(
                  64, 
                  &g_hPipes.hDspCodec[i]);  // DSP --> CODEC
      result = OtxDrvCreatePipe(
                  64, 
                  &g_hPipes.hCodecDsp[i]);  // CODEC --> DSP
      result = OtxDrvCreatePipe(
                  64, 
                  &g_hPipes.hDspDsp[i]);    // DSP --> DSP

    }

    for (i = 0; i < 2; i++) {
      // connect one end of the handset Pipes to DSPs
      result = OtxDrvConnectPipe(
                  g_hPipes.hDspCodec[i], 
                  OTX_PIPE_INPUT, 
                  g_hPhysDevs.hDsp[i], 
                  -1, 
                  NULL, 
                  0); 
      result = OtxDrvConnectPipe(
                  g_hPipes.hCodecDsp[i], 
                  OTX_PIPE_OUTPUT, 
                  g_hPhysDevs.hDsp[i], 
                  -1, 
                  NULL, 
                  0); 
          
    } 

    // connect logical devices within the DSPs
    for (i = 0; i < 2; i++) {
      // transmit direction: Tone Generator --> Codec
      result = OtxDrvConnectLogicalDevice(
                  g_hLogicDevs.hToneGenerator[i], 
                  -1, 
                  g_hPipes.hDspCodec[i], 
                  -1); 
    }
  }

  for (i = 0; i < g_demoEnv.iDspNo; i++) {

    // initialize the Tone Effect Generators
    result = OtxToneEffectsGeneratorInit(
                g_hLogicDevs.hToneGenerator[i], 
                OTX_DATA_FORMAT_COMPAND8_U_LAW, 
                OTX_TONE_SINEWAVE, 
                OTX_TONE_GAIN_0DB); 
  }

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_EFFECTS_GENERATOR>
    <nl><t OtxDataFormatE>
    <nl><t OtxToneEffectE>
    <nl><t OtxToneGainE>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>
    <nl><f OtxToneDtmfDial>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneEffectsGeneratorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Generated Tone
  IN OtxToneEffectE eToneType,        // @parm Type of tone effect to be generated on the channel
  IN OtxToneGainE   eToneGain         // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Set the custom parameters for each sinewave component to be generated by 
    the digital sinewave generator. A multiple frequency sinewave tone is 
    specified by using multiple elements in the <p sqSineWaveData> array. 

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |
    
    for (i = 0; i < 2; i++) {
      
      // create Sinewave Generator Logical Devices
      result = OtxDrvCreateLogicalDevice(
                  g_hPhysDevs.hDsp[i], 
                  OTX_LDEVICE_TONE_SINEWAVE_GENERATOR, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hSineWaveSender[i]); 
    } 
    
    // pipe creation and connection between DSP and Li, see example on 
    @ex <f OtxDrvConnectPipe> |

    // connect logical devices within the DSP
    for (i = 0; i < g_demoEnv.coDsp; i++) {
                  
      // transmit direction:
      // SineWaveGenerator --> Li
      result = OtxDrvConnectLogicalDevice(
                  g_hLogicDevs.hSineWaveSender[i], 
                  -1, 
                  g_hPipes.hDspLiSineWaveSender[i], 
                  -1);
    }

    // use the SineWaveGenerator device to send a tone (a DTMF tone) on current Line
    {
      static OTX_INT32 nCurrentLi = 0;

      OTX_RESULT result;
      static OTX_INT32 nDtmfDigit = 0;

      // Initialize the SineWave Sender 
      OtxToneSineWaveDataS sqSineWaveData[5];
  
      sqSineWaveData[0].nAttenuation = -9; // -9dB is about 25% of maximum amplitude
      sqSineWaveData[0].nPhase       = 0;  
  
      sqSineWaveData[1].nAttenuation = -9; // -9dB is about 25% of maximum amplitude
      sqSineWaveData[1].nPhase       = 0;  
  
      switch (nDtmfDigit) {
      case 0:
        // DTMF digit 0 
        sqSineWaveData[0].nFrequency   = 941;  // in Hz
        sqSineWaveData[1].nFrequency   = 1336; // in Hz       
        break;
      case 1:
        // DTMF digit 1 
        sqSineWaveData[0].nFrequency   = 697;  // in Hz
        sqSineWaveData[1].nFrequency   = 1209; // in Hz       
        break;
      case 2:
        // DTMF digit 2 
        sqSineWaveData[0].nFrequency   = 697;  // in Hz
        sqSineWaveData[1].nFrequency   = 1336; // in Hz       
        break;
      case 3:
        // DTMF digit 3 
        sqSineWaveData[0].nFrequency   = 697;  // in Hz
        sqSineWaveData[1].nFrequency   = 1477; // in Hz       
        break;
      case 4:
        // DTMF digit 4 
        sqSineWaveData[0].nFrequency   = 770;  // in Hz
        sqSineWaveData[1].nFrequency   = 1209; // in Hz       
        break;
      case 5:
        // DTMF digit 5 
        sqSineWaveData[0].nFrequency   = 770;  // in Hz
        sqSineWaveData[1].nFrequency   = 1336; // in Hz       
        break;
      case 6:
        // DTMF digit 6
        sqSineWaveData[0].nFrequency   = 770;  // in Hz
        sqSineWaveData[1].nFrequency   = 1477; // in Hz       
        break;
      case 7:
        // DTMF digit 7 
        sqSineWaveData[0].nFrequency   = 852;  // in Hz
        sqSineWaveData[1].nFrequency   = 1209; // in Hz       
        break;
      case 8:
        // DTMF digit 8 
        sqSineWaveData[0].nFrequency   = 852;  // in Hz
        sqSineWaveData[1].nFrequency   = 1336; // in Hz       
        break;
      case 9:
        // DTMF digit 9
        sqSineWaveData[0].nFrequency   = 852;  // in Hz
        sqSineWaveData[1].nFrequency   = 1477; // in Hz       
        break;
      }
  
      // initializing SineWave Generator
      result = OtxToneSineWaveGeneratorInit(
                  g_hLogicDevs.hSineWaveSender[nCurrentLi], 
                  g_demoEnv.law, 
                  sqSineWaveData,
                  2, // coSineWaveData
                  OTX_TONE_GAIN_0DB); 
  
  
      // enable SineWave Generator
      result = OtxDrvEnable(
                  g_hLogicDevs.hSineWaveSender[nCurrentLi],  
                  OTX_TASK_SYNC, 5000L);
    }

  @xref
    <e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SINEWAVE_GENERATOR>
    <nl><t OtxDataFormatE>
    <nl><t OtxToneSineWaveDataS>
    <nl><t OtxToneGainE>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>
    <nl><f OtxToneDtmfDial>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneSineWaveGeneratorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Generated Sinewave
  IN OtxToneSineWaveDataS sqSineWaveData[], // @parm Array of Sine Wave data (each element specifies one sine wave (frequency and gain))
  IN OTX_INT32      coSineWaveData,   // @parm Number of elements in <p sqSineWaveData> (1-5)
  IN OtxToneGainE   eToneGain         // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Initialize the DTMF generator. The DTMF generator will generate
    one DTMF tone infinitely.

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><t OtxDataFormatE>
    <nl><t OtxToneGainE>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DTMF_GENERATOR>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>
    <nl><f OtxToneDtmfDial>
    <nl><f OtxToneSequenceGeneratorInit>

  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneDtmfGeneratorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Generated DTMF digit
  IN OTX_UINT8      nDigit,           // @parm Digit to be generated
  IN OtxToneGainE   eToneGain         // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Initialize the DTMF sequence generator. The time between tone can be 
    specified in the 'nSilence' parameter, and the duration of each tone 
    is determined by the valued of the 'nDuration' parameter.

  @comm
    To set the number to be dialled, call function <f OtxToneDtmfDial>

    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |

    // create DTMF logical devices
    {
      OTX_RESULT result;
      OTX_INT32  i;

      for (i = 0; i < 2; i++) {
        // create Dtmf Detector Logical Devices
        result = OtxDrvCreateLogicalDevice( 
                    g_hPhysDevs.hDsp[i], 
                    OTX_LDEVICE_TONE_DTMF_DETECTOR, 
                    g_hEventQueue, 
                    i, 
                    &g_hLogicDevs.hDtmfDetector[i]); 

        // create DTMF Dialers Logical Devices
        result = OtxDrvCreateLogicalDevice(
                    g_hPhysDevs.hDsp[i], 
                    OTX_LDEVICE_TONE_DTMF_DIALER, 
                    g_hEventQueue, 
                    i, 
                    &g_hLogicDevs.hDtmfDialler[i]); 

      }
    }
    
    // connect devices
    {
      OTX_INT32 nLi;
      OTX_INT32 nDsp;
      OTX_INT32 i;
      OTX_RESULT result;
      OTX_UINT8 sqTimeSlotMask[32];
      OTX_INT32 ts;

      for (nDsp = 0; nDsp < 2; nDsp++) {
        result = OtxDrvCreatePipe(
                    64, 
                    &g_hPipes.hLiDspDtmfDetector[i]); 
        result = OtxDrvCreatePipe(
                    64, 
                    &g_hPipes.hDspLiDtmfDialer[i]); 
      }                  

      // connect pipes for the DTMF Dialer and Detector
      for (nLi = 0; nLi < 2; nLi++) {
    
        // Create two pipes per Li. One pipe to each DSP
        for (i = 0; i < 2; i++) {

          nDsp = nLi;

          // Connect Pipes
          ts = 1+i;
          memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
          sqTimeSlotMask[ts] = 0xFF;

          result = OtxDrvConnectPipe(
                      g_hPipes.hLiDspDtmfDetector[nDsp], 
                      OTX_PIPE_INPUT,  
                      g_hPhysDevs.hLi[nLi],  
                      -1, 
                      sqTimeSlotMask, 
                      32); 
          result = OtxDrvConnectPipe(
                      g_hPipes.hDspLiDtmfDialer[nDsp], 
                      OTX_PIPE_OUTPUT, 
                      g_hPhysDevs.hLi[nLi],  
                      -1, 
                      sqTimeSlotMask, 
                      32); 

          result = OtxDrvConnectPipe(
                      g_hPipes.hLiDspDtmfDetector[nDsp], 
                      OTX_PIPE_OUTPUT, 
                      g_hPhysDevs.hDsp[nDsp], 
                      -1, 
                      NULL, 
                      0); 
          result = OtxDrvConnectPipe(
                      g_hPipes.hDspLiDtmfDialer[nDsp], 
                      OTX_PIPE_INPUT,  
                      g_hPhysDevs.hDsp[nDsp], 
                      -1, 
                      NULL, 
                      0); 
        }
      }
    } // connect devices

    // connect logical devices in DSP
    for (i = 0; i < 2; i++) {

      // make the following connections in receive direction:
      // Li ---> DtmfDetector
      result = OtxDrvConnectLogicalDevice(
                  g_hPipes.hLiDspDtmfDetector[i],     
                  -1, 
                  g_hLogicDevs.hDtmfDetector[i],       
                  -1); 

      // make the following connections in transmit direction:
      // DtmfDialler   -----> Li
      result = OtxDrvConnectLogicalDevice(
                  g_hLogicDevs.hDtmfDialler[i], 
                  -1, 
                  g_hPipes.hDspLiDtmfDialer[i], 
                  -1); 
    }

    for (i = 0; i < 2; i++) {
      // initialize DTMF logical devices

      // Initialize the DTMF Detector
      result = OtxToneDtmfDetectorInit(
                  g_hLogicDevs.hDtmfDetector[i], 
                  OTX_DATA_FORMAT_COMPAND8_A_LAW, 
                  OTX_TRUE); 

      // Initialize the DTMF Diallers
      result = OtxToneDtmfDiallerInit(
                  g_hLogicDevs.hDtmfDialler[i], 
                  OTX_DATA_FORMAT_COMPAND8_A_LAW, 
                  8, 
                  8, 
                  OTX_TONE_GAIN_0DB); 

      // enable DTMF logical device
      result = OtxDrvEnable(
                  g_hLogicDevs.hDtmfDetector[i],  
                  OTX_TASK_SYNC, 
                  5000L); 
      result = OtxDrvEnable(
                  g_hLogicDevs.hDtmfDialler[i],  
                  OTX_TASK_SYNC, 
                  5000L); 
    }


  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DTMF_DIALER>
    <nl><t OtxDataFormatE>
    <nl><t OtxToneGainE>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDial>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneDtmfDiallerInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Generated DTMF digits
  IN OTX_INT32      nOnTime,          // @parm Duration of DTMF tones in milliseconds, 12.75 milliseconds resolution (0 for infinity)
  IN OTX_INT32      nOffTime,         // @parm Silence time between of DTMF tones in increments of 12.75ms
  IN OtxToneGainE   eToneGain         // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Set the number to be dialled by the DTMF dialler.

  @comm
    Start dialling by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |

    char *g_sqNumber0 = "0123456789ABCD*#";
    OTX_INT32 nDigits = strlen(g_sqNumber0);
    OTX_HANDLE hHandle = g_hLogicDevs.hDtmfDialler[nCurrentLi];

    // sending digits on current Line, will receive Event when completed
    result = OtxToneDtmfDial(
                hHandle, 
                (OTX_UINT8 *)g_sqNumber0, 
                nDigits, 
                DEMO_TASK_REF, 
                OTX_TIME_INFINITY); 

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DTMF_DIALER>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>
    <nl><f OtxToneSequenceGeneratorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneDtmfDial(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OTX_UINT8      sqDigits[],       // @parm Digits to send
  IN OTX_INT32      nDigits,          // @parm Number of digits to send
  IN OTX_TASK_REF   nTaskRef,         // @parm See <t OTX_TASK_REF>
  IN OTX_TIME       nWaitMaxMs        // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Initialize the Single Frequency sequence generator. The time between tones 
    can be specified in the 'nOffTime' parameter, and the duration of each 
    tone is determined by the valued of the 'nOntime' parameter.

  @comm
    Start generation by calling the function <f OtxToneFrequencySequencySend>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><t OtxDataFormatE>
    <nl><t OtxToneGainE>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SEQUENCE_GENERATOR>
    <nl><f OtxToneFrequencySequencySend>
    <nl><f OtxToneConstValGeneratorInit>
    <nl><f OtxToneEffectsGeneratorInit>
    <nl><f OtxToneSineWaveGeneratorInit>
    <nl><f OtxToneDtmfGeneratorInit>
    <nl><f OtxToneDtmfDiallerInit>

  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneSequenceGeneratorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Generated DTMF digits
  IN OTX_TIME       nOnTime,          // @parm Length of (tones) in milliseconds
  IN OTX_TIME       nOffTime,         // @parm Delay between (tones) in milliseconds
  IN OtxToneGainE   eToneGain         // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Send a frequency sequence.

  @comm
    Set the duration lengths and the gain of the tone by calling the
    function <f OtxToneSequenceGeneratorInit>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SEQUENCE_GENERATOR>
    <nl><f OtxToneSequenceGeneratorInit>
    <nl><f OtxToneDtmfDial>

  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneSequenceSend(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OTX_INT32      sqFreqs[],        // @parm Frequencies to send in Hz
  IN OTX_INT32      nTones,           // @parm Number of Frequencies (tones) to send
  IN OTX_TASK_REF   nTaskRef,         // @parm See <t OTX_TASK_REF>
  IN OTX_TIME       nWaitMaxMs        // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Configures the Silence Detector. A silence is detected when a
    certain number of received samples have a combined energy level of
    less than the specified threshold.

  @comm
    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |

      // create silence Detector
    result = OtxDrvCreateLogicalDevice(
                g_hPhysDevs.hDsp, 
                OTX_LDEVICE_TONE_SILENCE_DETECTOR, 
                g_hEventQueue, 
                0, 
                &g_hLogicDevs.mSilenceDetector.hHandle); 

    // pipe connection between DSP and CODEC, see example on
    @ex <f OtxDrvConnectPipe> |


    // connect logical device
    {
      OTX_HANDLE hPipe;

      result = OtxDrvCreatePipe(
                  64, 
                  &hPipe); 

      memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
      sqTimeSlotMask[6] = 0xFF;
      result = OtxDrvConnectPipe(
                  hPipe, 
                  OTX_PIPE_INPUT,  
                  g_hPhysDevs.hLi[0],  
                  -1, 
                  sqTimeSlotMask, 
                  32); 
      result = OtxDrvConnectPipe(
                  hPipe, 
                  OTX_PIPE_OUTPUT, 
                  g_hPhysDevs.hDsp,  
                  -1, 
                  sqTimeSlotMask, 
                  32); 
        
      // connect the silence detector as the sink of this connection
      result = OtxDrvConnectLogicalDevice(
                  hPipe, 
                  -1, 
                  g_hLogicDevs.mSilenceDetector.hHandle, 
                  -1); 
    }

    // Initialize Silence Detector
    result = OtxToneSilenceDetectorInit(
                g_hLogicDevs.mSilenceDetector.hHandle, 
                OTX_DATA_FORMAT_COMPAND8_A_LAW, 
                2, 
                2000); 

    // enable Silence Detector (always on)
    result = OtxDrvEnable(
                g_hLogicDevs.mSilenceDetector.hHandle,  
                OTX_TASK_SYNC, 
                5000L); 

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><t OtxDataFormatE>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SILENCE_DETECTOR>
    <nl><f OtxToneSilenceDetectorStatus>
    <nl><f OtxToneDtmfDetectorInit>
    <nl><f OtxToneFskDetectorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneSilenceDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Monitored Data Stream
  IN OTX_INT32      nSilenceDuration, // @parm Time (in 12.75ms increments) that silence must be present to be reported
  IN OTX_INT32      nThreshold        // @parm Low thresHold stipulates very silent silence for silence to be reported;
                                      // Higher thresHold accepts noise in the silence and silence is still being reported
                                      // Typical value to be used: 4000
);


/*******************************************************************************

  @func
    Reports whether the silence detector has detected silence or a tone

  @ex |

    OTX_BOOL  bSilence;      
      
    result = OtxToneSilenceDetectorStatus(
                g_hLogicDevs.mSilenceDetector.hHandle, 
                &bSilence); 
      
    // display the status: OTX_TRUE if silence, OTX_FALSE if tone detected
    if (bSilence) {
      DemoLogf("s");
    } else {
      DemoLogf(".");
    }

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SILENCE_DETECTOR>
    <nl><f OtxToneSilenceDetectorInit>
    <nl><f OtxToneDtmfDetectorStatus>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneSilenceDetectorStatus(
  IN  OTX_HANDLE    hDevice,          // @parm Handle to the Logical device
  OUT OTX_BOOL      *bSilence         // @parm OTX_TRUE if silence, OTX_FALSE if tone detected
);


/*******************************************************************************

  @func
    Configures the Dial Tone Detector. A dial tone is detected when a
    dial tone has been present for the time period specied with parameter
    nToneDuration.

  @comm
    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><t OtxDataFormatE>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DIALTONE_DETECTOR>
    <nl><f OtxToneDialtoneDetectorStatus>
    <nl><f OtxToneDtmfDetectorInit>
    <nl><f OtxToneFskDetectorInit>

  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneDialtoneDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Monitored Data Stream
  IN OTX_INT32      nToneDuration     // @parm Time (in 12.75ms increments) that dialtone must be present to be reported
);


/*******************************************************************************

  @func
    Reports whether the Dial Tone Detector has detected a dial tone or not

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DIALTONE_DETECTOR>
    <nl><f OtxToneDialtoneDetectorInit>
    <nl><f OtxToneDtmfDetectorStatus>
    <nl><f OtxToneSilenceDetectorStatus>

  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneDialtoneDetectorStatus(
  IN  OTX_HANDLE    hDevice,          // @parm Handle to the Logical device
  OUT OTX_BOOL      *bDialtone        // @parm OTX_TRUE if Dial Tone present, OTX_FALSE if not
);


/*******************************************************************************

  @func
    Configures the DTMF Detector.

  @comm
    The received digits can either be stored in the Event Queue and be
    extracted with the appropriate Read function, or they can be stored 
    internally in which case only the most recently received digit can be 
    retrieved with a call to function <f OtxToneDtmfDetectorStatus>

    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  see example on <f OtxToneDtmfDiallerInit>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DTMF_DETECTOR>
    <nl><t OtxDataFormatE>
    <nl><f OtxToneDtmfDetectorStatus>
    <nl><f OtxToneSilenceDetectorInit>
    <nl><f OtxToneDialtoneDetectorInit>
    <nl><f OtxToneFskDetectorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneDtmfDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Monitored Data Stream
  IN OTX_BOOL       bStoreInQueue     // @parm If OTX_TRUE, generate events and store the received digits
                                      // in the Event QueusFifo, If OTX_FALSE Don't store. Provide polling test
                                      // with function <f OtxToneDtmfDetectorStatus>
);


/*******************************************************************************

  @func
    This function allows the user to poll and check for the most recently
    received DTMF tone. If a tone was received, the function returns OTX_S_OK 
    and the last digit is in the cDtmfDigit parameter. If no tone was detected, 
    the function returns OTX_S_FALSE.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DTMF_DETECTOR>
    <nl><f OtxToneDtmfDetectorInit>
    <nl><f OtxToneSilenceDetectorStatus>

  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneDtmfDetectorStatus(
  IN  OTX_HANDLE    hDevice,          // @parm Handle to the Logical device
  OUT char          *cDtmfDigit       // @parm Most recent DTMF digit detected (or \0 if none)
);


/*******************************************************************************

  @func
    Initialize the MF sequence generator. The time between tone can be specified
    in the 'nSilence' parameter, and the duration of each tone is determined 
    by the valued of the 'nDuration' parameter.

  @comm
    To set the number to be dialled, call function <f OtxToneMfDial>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_MF_DIALER>
    <nl><t OtxDataFormatE>
    <nl><t OtxMfToneE>
    <nl><t OtxToneGainE>
    <nl><f OtxToneMfDial>

  @ex |
    // createLogicalDevices(
    for (i = 0; i < 2; i++) {
      // create Sinewave Receiver (MF Tone Detector) logical devices 
      result = OtxDrvCreateLogicalDevice(
                  g_hPhysDevs.hDsp[i], 
                  OTX_LDEVICE_TONE_MF_DETECTOR, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hSineWaveReceiver[i]); 

      // create MF Dialers Logical Devices
      result = OtxDrvCreateLogicalDevice(
                  g_hPhysDevs.hDsp[i], 
                  OTX_LDEVICE_TONE_MF_DIALER, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hMfDialler[i]); 

    }

    // connect devices(
    {
      OTX_INT32 nLi;
      OTX_INT32 nDsp;
      OTX_INT32 i;
      OTX_RESULT result;
      OTX_UINT8 sqTimeSlotMask[32];
      OTX_INT32 ts;

      // connect HDLC pipes 
      for (i = 0; i < 2; i++) {
        // Create an incoming and outgoing Pipe representing the external T1/E1
        result = OtxDrvCreatePipe(
                    64, 
                    &g_hPipes.hLiIn[i]); 
        result = OtxDrvCreatePipe(
                    64, 
                    &g_hPipes.hLiOut[i]);

        // Connect external T1/E1 pipes. Only connect one end of each pipe. 
        if (g_demoEnv.eT1E1Mode == OTX_T1E1_MODE_T1) {
          // T1 mode
          ts = 24; 
        } else {
          // E1 mode
          ts = 16; 
        }
        memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
        sqTimeSlotMask[ts] = 0xFF;
        result = OtxDrvConnectPipe(
                    g_hPipes.hLiIn[i], 
                    OTX_PIPE_OUTPUT, 
                    g_hPhysDevs.hLi[i], 
                    1, 
                    sqTimeSlotMask, 32); 
        result = OtxDrvConnectPipe(
                    g_hPipes.hLiOut[i], 
                    OTX_PIPE_INPUT, 
                    g_hPhysDevs.hLi[i], 
                    1, 
                    sqTimeSlotMask, 
                    32); 

        // Connect HDLC Sender and Receiver to the external T1/E1 Pipes
        result = OtxDrvConnectLogicalDevice(
                    g_hPipes.hLiIn[i], 
                    -1, 
                    g_hLogicDevs.hHdlcReceiver[i], 
                    -1); 
        result = OtxDrvConnectLogicalDevice(
                    g_hLogicDevs.hHdlcSender[i], 
                    -1, 
                    g_hPipes.hLiOut[i], 
                    -1); 
      }
    }

    // connect logical devices in DSP
    for (i = 0; i < 2; i++) {
      // create pipes 
      result = OtxDrvCreatePipe(
                  64, 
                  &g_hPipes.hLiDspSineWaveReceiver[i]); 
      result = OtxDrvCreatePipe(
                  64, 
                  &g_hPipes.hDspLiSineWaveSender[i]); 
      
      // make the following connections in receive direction:
      result = OtxDrvConnectLogicalDevice(
                  g_hPipes.hLiDspSineWaveReceiver[i], 
                  -1, 
                  g_hLogicDevs.hSineWaveReceiver[i],   
                  -1);

      // make the following connections in transmit direction:
      result = OtxDrvConnectLogicalDevice(
                  g_hLogicDevs.hMfDialler[i],      
                  -1, 
                  g_hPipes.hDspLiSineWaveSender[i], 
                  -1); 

    }

    for (i = 0; i < 2; i++) {
      // initialize MF logical devices
      
      // initialize the SineWave Receiver (MF Tone Detector)
      result = OtxToneMfDetectorInit(
                  g_hLogicDevs.hSineWaveReceiver[i], 
                  OTX_DATA_FORMAT_COMPAND8_A_LAW,
                  OTX_TONE_MF_R1,  
                  OTX_TRUE,           
                  OTX_FALSE,          
                  25);                

      // initialize the MF Diallers
      result = OtxToneMfDiallerInit(
                  g_hLogicDevs.hMfDialler[i], 
                  OTX_DATA_FORMAT_COMPAND8_A_LAW, 
                  OTX_TONE_MF_R1,
                  50,         
                  25, 
                  OTX_TONE_GAIN_0DB); 

      // enable MF logical device
      result = OtxDrvEnable(
                  g_hLogicDevs.hSineWaveReceiver[i],  
                  OTX_TASK_SYNC, 
                  5000L); 
      result = OtxDrvEnable(
                  g_hLogicDevs.hMfDialler[i],  
                  OTX_TASK_SYNC, 
                  5000L); 
    }


*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneMfDiallerInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Generated DTMF digits
  IN OtxMfToneE     eMfToneType,      // @parm Type of MF Tone to detect (see <t OtxMfToneE>)
  IN OTX_INT32      nOnTime,          // @parm Duration of MF tones in milliseconds, 12.75 milliseconds resolution (0 for infinity)
  IN OTX_INT32      nOffTime,         // @parm Silence time between of MF tones in milliseconds, 12.75 milliseconds resolution
  IN OtxToneGainE   eToneGain         // @parm Digital gain of generated tone
);


/*******************************************************************************

  @func
    Dial a string of MF digits (tones).

  @comm
    This function is a non-blocking function. The event 
    <e OtxMfDialerEventCodesE.OTX_MF_DIALER_EC_COMPLETE> is generated when the 
    device has completed dialing the specified string of MF digits.

  @ex |
      static OTX_INT32 nCurrentLi = 0;      
      OTX_HANDLE hHandle = g_hLogicDevs.hMfDialler[nCurrentLi];
      char *sqDigitString;
        
      switch (g_demoEnv.eMfToneType) {
        case OTX_TONE_MF_R1:
          sqDigitString = "0123456789abcde";
          break;
        case OTX_TONE_MF_R2_FORWARD:
        case OTX_TONE_MF_R2_BACKWARD:
          sqDigitString = "123456789abcdef";
          break;
        default: 
          sqDigitString = "1";
          break;
       }
        
      // dial MF digit string, will receive Event when completed
      result = OtxToneMfDial(
                  hHandle, 
                  (OTX_UINT8 *)sqDigitString, 
                  strlen(sqDigitString), 
                  DEMO_TASK_REF, 
                  OTX_TIME_INFINITY); 

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_MF_DIALER>
    <nl><f OtxToneMfDiallerInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneMfDial(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OTX_UINT8      sqDigits[],       // @parm Digits/Tones (see <t OtxMfToneE>) to send. The digits 
                                      // can either be specified as a numeric value (0-15), or as 
                                      // characters '0' through '9' and 'a' through 'f' (where 'a' 
                                      // through 'f' represents 10 through 15). Please not that the R1 
                                      // digits are in the range of 0-14, and the R2 digits are in the 
                                      // range of 1-15.
  IN OTX_INT32      nDigits,          // @parm Number of digits/tones to send
  IN OTX_TASK_REF   nTaskRef,         // @parm See <t OTX_TASK_REF>
  IN OTX_TIME       nWaitMaxMs        // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Configures the MF Detector.

  @comm
    The received digits can either be stored in the Event Queue and be
    extracted with the appropriate Read function, or they can be stored 
    internally in which case only the most recently received digit can be 
    retrieved with a call to function <f OtxToneMfDetectorStatus>

    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  see example on <f OtxToneMfDiallerInit> 

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_MF_DETECTOR>
    <nl><t OtxDataFormatE>
    <nl><t OtxMfToneE>
    <nl><f OtxToneMfDetectorStatus>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneMfDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Monitored Data Stream
  IN OtxMfToneE     eMfToneType,      // @parm Type of MF Tone to detect (see <t OtxMfToneE>)
  IN OTX_BOOL       bStoreInQueue,    // @parm If OTX_TRUE, generate events and store the received digits
                                      // in the Event QueusFifo, If OTX_FALSE Don't store. Provide polling test
                                      // with function <f OtxToneMfDetectorStatus>
  IN OTX_BOOL       bReportPause,     // @parm If OTX_TRUE the detector generates an event when a tone ends
  IN OTX_INT16      nMinDuration      // @parm Minimum duration required to validate a digit
);


/*******************************************************************************

  @func
    This function allows the user to poll and check for the most recently
    received MF tone. If a tone was received, the function returns 
    OTX_S_OK and the last digit is in the cDigit parameter. If no tone was 
    detected, the function returns OTX_S_FALSE.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_MF_DETECTOR>
    <nl><f OtxToneMfDetectorInit>
  
  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneMfDetectorStatus(
  IN  OTX_HANDLE    hDevice,          // @parm Handle to the Logical device
  OUT char          *nDigit           // @parm Most recent digit detected (or -1 if none)
);


/*******************************************************************************

  @func
    Configures the Dual Tone Detector.

  @comm
    The dual tone detector can detect a number tone pairs (two-frequency tones). 
    The frequencies to detect are supplied as a parameter to this function. 
    This device can handle 6 frequencies (FreqA through FreqF). When a certain 
    dual tone is detected, an event is generated and the detected dual tone is 
    coded according to the table below.

  @ex Dual Tone Coding Matrix |

          .         FreqA	 FreqB  FreqC  FreqD  FreqE	 FreqF 
          FreqA    { -1,    1,      2,     4,     7,    11  },
          FreqB    {  1,   -1,      3,     5,     8,    12  },
          FreqC    {  2,    3,     -1,     6,     9,    13  },
          FreqD    {  4,    5,      6,    -1,    10,    14  },
          FreqE    {  7,    8,      9,    10,    -1,    15  },
          FreqF    { 11,   12,     13,    14,    15,    -1  }

   @comm 

    The Dual Tone detector device is highly configurable to be able to detect 
    various tones of various amplitudes. This is a brief description of 
    how the device works:

    <nl> 
         1. The device gathers 102 samples of tone data (12.75 ms). 

    <nl> 
         2. It then runs an algorithm for each of the 6 frequencies that the 
            device can work with. The algorithm will produce energy numbers for each 
            frequency. If the buffer of 102 samples contains a frequency component 
            which closely matches one of the detection frequencies it will produce 
            a high energy number for that frequency. If there is a mismatch in frequencies 
            then the energy number will be low.

    <nl>
         3. The code now has 6 energy numbers to work with. It will now run these 
            numbers through a series of tests to see if they pass the thresholds (which 
            can be specified through attributes through API functions). The first test 
            performed is the 
            <e OtxDualDetectorAttributesE.OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TONE_ENERGY>
            threshold. The two highest energy values have to be higher than this threshold 
            to pass as a detected tone. The default value for this threshold is 900. 
            Lower values will make the detection criteria less stringent.

    <nl>
         4. The next test is the 
            <e OtxDualDetectorAttributesE.OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TWIST>
            threshold. The code will look at the two highest energy levels and calculate 
            their ratio (MaxEnergy/2ndMaxEnergy*100). The ratio must be less than this 
            threshold to pass as a detected tone. The default value for this threshold is 150 (1.5 times). 
            Higher values will make the detection criteria less stringent.

    <nl>
         5. The next test is the 
            <e OtxDualDetectorAttributesE.OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_RELATIVE_3RD> 
            threshold. The code has already looked at the two highest energy levels. It will 
            now look at the third highest energy level. It will calculate the ration 
            between the 2nd highest and the 3rd highest (2ndMaxEnergy/3rdMaxEnergy*100). The 
            ratio must be greater than this threshold to pass as a detected tone. The 
            default value for this threshold is 150 (1.5 times). Lower values will make the detection criteria less stringent.

    <nl> 
         6. The last check performed is the duration test. The duration paramter is supplied 
            in this init function. The detection has to generate a positive and consistent 
            result for a number of 12.75 ms buffers in order for the code to generate a 
            detection event to the API.


    <nl> If the symptom is that you are not the correct detection events, it might be necessary to 
    adjust the thresholds for the device. To further analyze which threshold that need to be 
    adjusted, the 
    <e OtxDualDetectorAttributesE.OTX_ATTR_DUAL_TONE_DETECTOR_VERBOSE_STATUS> attribute can be 
    set to a value higher than zero, which causes the device to generate 
    <e OtxDualToneDetectorEventCodesE.OTX_DUAL_TONE_DETECTOR_EC_VERBOSE_STATUS> events. 
    Higher values generates more verbose debug information. The 
    m_nParam member of the OTX_DUAL_TONE_DETECTOR_EC_VERBOSE_STATUS event provides the following 
    information:


  @ex Verbose Status Information (verbose level 1) |

      Bit 15-0: Bit 31-16:                            Event occurs when
      ~~~~~~~~~ ~~~~~~~~~                             ~~~~~~~~~~~~~~~~~~
      2         n2ndMaxEnergy                         THRESHOLD_TONE_ENERGY test has failed to meet criteria	
      3         nMaxEnergy/n2ndMaxEnergy*100          THRESHOLD_TWIST test has failed to meet criteria	
      4         n2ndMaxEnergy/n3rdMaxEnergy*100       THRESHOLD_RELATIVE_3RD test has failed to meet criteria	
      5         nDualTone ID of detected dual tone    Duration consistency test failed. Another dual 
      .                                               tone was detected before the end of the required 
      .                                               duration.

  @ex Verbose Status Information (verbose level 5) |

     Bit 15-0: Bit 31-16:                             Event occurs when
     ~~~~~~~~~ ~~~~~~~~~                              ~~~~~~~~~~~~~~~~~~
     20        Energy level of tone index 0           once for every 12.75ms buffer
     21        Energy level of tone index 1           once for every 12.75ms buffer
     22        Energy level of tone index 2           once for every 12.75ms buffer
     23        Energy level of tone index 3           once for every 12.75ms buffer
     24        Energy level of tone index 4           once for every 12.75ms buffer
     25        Energy level of tone index 5           once for every 12.75ms buffer


   @comm 
   
    The detected tones can either be stored in the Event Queue and be
    extracted with the appropriate Read function, or they can be stored internally
    in which case only the most recently received digit can be retrieved with
    a call to function <f OtxToneDualDetectorStatus>

    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |
    // create Dula Tone Detector Logical Devices
    for (i = 0; i < 2; i++) {
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hDsp[i], 
                  OTX_LDEVICE_TONE_DUAL_DETECTOR, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hDualToneDetector[i]); 
    }

    // pipe connection between DSP and LI, see example on
    @ex <f OtxDrvConnectPipe> |

    // connect logical device
    for (i = 0; i < g_demoEnv.coDsp; i++) {
      // create the following connections in receive direction:
      //  Li --> DualToneDetector
      result = OtxDrvConnectLogicalDevice(
                  g_hPipes.hLiDspDualToneDetector[i],     
                  -1, 
                  g_hLogicDevs.hDualToneDetector[i],       
                  -1); 
    }

    // initialize Dual Tone Detector logical devices
    {
      OTX_RESULT result;
      OTX_INT32  i;
  
      OTX_UINT32 nThreshold;
      i = 0;
      
      // get current thresholds
      result = OtxDrvGetAttributeValueUint32(
                  g_hLogicDevs.hDualToneDetector[i], 
                  OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TONE_ENERGY, 
                  &nThreshold); 
      result = OtxDrvGetAttributeValueUint32(
                  g_hLogicDevs.hDualToneDetector[i], 
                  OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_PAUSE_ENERGY, 
                  &nThreshold); 
      result = OtxDrvGetAttributeValueUint32(
                  g_hLogicDevs.hDualToneDetector[i], 
                  OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TWIST, 
                  &nThreshold); 
      
      result = OtxDrvGetAttributeValueUint32(
                  g_hLogicDevs.hDualToneDetector[i], 
                  OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_RELATIVE_3RD, 
                  &nThreshold); 


      // Modifing default thresholds
      nThreshold = 600; // Min tone energy for first and second tones
      result = OtxDrvSetAttributeValueUint32(
                  g_hLogicDevs.hDualToneDetector[i], 
                  OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TONE_ENERGY, 
                  nThreshold); 
      
      nThreshold = 200;  // Max energy for pause
      result = OtxDrvSetAttributeValueUint32(
                  g_hLogicDevs.hDualToneDetector[i], 
                  OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_PAUSE_ENERGY, 
                  nThreshold); 
      
      nThreshold = 300; // Max ratio HighestEnergy/2ndHighestEnergy*100 (225 = 2.25 times)
      result = OtxDrvSetAttributeValueUint32(
                  g_hLogicDevs.hDualToneDetector[i], 
                  OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TWIST, 
                  nThreshold); 
      
      nThreshold = 30;  // Min ratio 2ndHighestEnergy/3rdHighestEnergy*100  (30 = 0.3 times)
      result = OtxDrvSetAttributeValueUint32(
                  g_hLogicDevs.hDualToneDetector[i], 
                  OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_RELATIVE_3RD, 
                  nThreshold); 
    
    
      for (i = 0; i < g_demoEnv.coDsp; i++) {
      
        OTX_UINT32    sqFreq[4];       // An array of up to 6 frequencies to be detected.
        OTX_INT32     coFreq = 4;      // Number of elements in <p sqFreq>
           

        // 350/440 is dial tone (dualtone index 1)
        // 440/480 is ring tone (dualtone index 3)
        // 480/620 is busy tone (dualtone index 6)

        // refer to "Dual Tone Coding Matrix"
        sqFreq[0] = 350;  // Freq A in Hz
        sqFreq[1] = 440;  // Freq B in Hz       
        sqFreq[2] = 480;  // Freq C in Hz
        sqFreq[3] = 620;  // Freq D in Hz       
             
        // Initialize the Generic Dual Tone Detector. Use a couple of DTMF tone frequencies as an example
        result =  OtxToneDualDetectorInit(
                    g_hLogicDevs.hDualToneDetector[i],   
                    g_demoEnv.law,    
                    sqFreq,           
                    coFreq,           
                    OTX_TRUE,         
                    OTX_TRUE,         
                    100);

        result = OtxDrvEnable(
                    g_hLogicDevs.hDualToneDetector[i],  
                    OTX_TASK_SYNC, 
                    5000L); 
    }


  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DUAL_DETECTOR>
    <nl><t OtxDataFormatE>
    <nl><f OtxToneDualDetectorStatus>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneDualDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Monitored Data Stream
  IN OTX_UINT32     sqFreq[],         // @parm An array of up to 6 frequencies to be detected.
  IN OTX_INT32      coFreq,           // @parm Number of elements in <p sqFreq>
  IN OTX_BOOL       bStoreInQueue,    // @parm If OTX_TRUE, generate events and store the received digits
  IN OTX_BOOL       bReportPause,     // @parm If OTX_TRUE the detector generates an event when a tone ends
  IN OTX_INT16      nMinDuration      // @parm Minimum duration required to validate a digit
);


/*******************************************************************************

  @func
    This function allows the user to poll and check for the most recently
    received dual tone. If a tone was received, the function returns 
    OTX_S_OK and the last code is in the cCode parameter. If no tone was 
    detected, the function returns OTX_S_FALSE.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_DUAL_DETECTOR>
    <nl><f OtxToneDualDetectorInit>

  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneDualDetectorStatus(
  IN  OTX_HANDLE    hDevice,          // @parm Handle to the Logical device
  OUT char          *nCode            // @parm Most recent dual tone code detected (or -1 if none)
);


/*******************************************************************************

  @func
    Configures the Single Tone Detector.

  @comm
    The Single tone detector can detect a Single tone. 
    The frequencies to detect are supplied as a parameter to this function. 
    This device can handle 6 frequencies (FreqA through FreqF). When a certain 
    single tone is detected, an event is generated and the detected single 
    tone is coded according to the table below.

  @ex Single Tone Coding Matrix |

          FreqA    {0}
          FreqB    {1}
          FreqC    {2}
          FreqD    {3}
          FreqE    {4}
          FreqF    {5}

  @comm 
   
    The detected tones can either be stored in the Event Queue and be
    extracted with the appropriate Read function, or they can be stored internally
    in which case only the most recently received digit can be retrieved with
    a call to function <f OtxToneSingleDetectorStatus>

    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

   @comm 

    The Single Tone detector device is highly configurable to be able to detect 
    various tones of various amplitudes. This is a brief description of 
    how the device works:

    <nl>
         1. The device gathers 102 samples of tone data (12.75 ms). 
         
    <nl>
         2. It then runs an algorithm for each of the 6 frequencies that the 
            device can work with. The algorithm will produce energy numbers for each 
            frequency. If the buffer of 102 samples contains a frequency component 
            which closely matches one of the detection frequencies it will produce 
            a high energy number for that frequency. If there is a mismatch in frequencies 
            then the energy number will be low.

    <nl>
         3. The code now has 6 energy numbers to work with. It will now run these 
            numbers through a series of tests to see if they pass the thresholds (which 
            can be specified through attributes through API functions). The first test 
            performed is the 
            <e OtxSingleDetectorAttributesE.OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_TONE_ENERGY>
            threshold. The two highest energy values have to be higher than this threshold 
            to pass as a detected tone. The default value for this threshold is 1000. 
            Lower values will make the detection criteria less stringent.

    <nl>
         4. The next check performed is the duration test. The duration paramter is supplied 
            in this init function. The detection has to generate a positive and consistent 
            result for a number of 12.75 ms buffers in order for the code to generate a 
            detection event to the API.

    <nl>
         5. The last test is the 
            <e OtxSingleDetectorAttributesE.OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_VARIANCE_ENERGY> 
            threshold. The code has already found the highest energy level. This threshold
            determines how much the energy level is allow to vary from one 12.75ms buffer
            to the next. The code will look at the current energy level and compare to the
            average energy level detected so far. The default value for this threshold is 
            200. Higher values will make the detection criteria less stringent.


    <nl> If noticing symptoms of incorrect detection events, it might be necessary to 
    adjust the thresholds for the device. To further analyze which threshold that need to be 
    adjusted, the 
    <e OtxSingleDetectorAttributesE.OTX_ATTR_SINGLE_TONE_DETECTOR_VERBOSE_STATUS> attribute can be 
    set to a value higher than zero, which causes the device to generate 
    <e OtxSingleToneDetectorEventCodesE.OTX_SINGLE_TONE_DETECTOR_EC_VERBOSE_STATUS> events. 
    Higher values generates more verbose debug information. The 
    m_nParam member of the OTX_SINGLE_TONE_DETECTOR_EC_VERBOSE_STATUS event provides the following 
    information:


  @ex Verbose Status Information (verbose level 1) |

      Bit 15-0: Bit 31-16:                     Event occurs when
      ~~~~~~~~~ ~~~~~~~~~                      ~~~~~~~~~~~~~~~~~~
      2         n2ndMaxEnergy                  THRESHOLD_TONE_ENERGY test has failed to meet criteria	
      5         Index of detected dual tone    Duration consistency test failed. Another dual 
      6         nMaxEnergy                     THRESHOLD_VARIANCE test has failed to meet criteria. The
      .                                        energy level varied beyond the threshold (when compared
      .                                        to the average energy level detected so far.

  @ex Verbose Status Information (verbose level 5) |

      Bit 15-0: Bit 31-16:                     Event occurs when
      ~~~~~~~~~ ~~~~~~~~~                      ~~~~~~~~~~~~~~~~~~
      20        Energy level of tone index 0   for every 12.75ms buffer
      21        Energy level of tone index 1   for every 12.75ms buffer
      22        Energy level of tone index 2   for every 12.75ms buffer
      23        Energy level of tone index 3   for every 12.75ms buffer
      24        Energy level of tone index 4   for every 12.75ms buffer
      25        Energy level of tone index 5   for every 12.75ms buffer


  @ex |
    for (i = 0; i < 2; i++) {
      // Create Single Tone Detector Logical Devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hDsp[i], 
                  OTX_LDEVICE_TONE_SINGLE_DETECTOR, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hSingleToneDetector[i]); 
    }

    // pipe connection between DSP and LI, see example on
    @ex <f OtxDrvConnectPipe> |

    // connect logical device
    for (i = 0; i < 2; i++) {           
      // create the following Connections in receive direction:
      // Li --> SingleToneDetector
      result = OtxDrvConnectLogicalDevice(
                  g_hPipes.hLiDspSingleToneDetector[i],     
                  -1, 
                  g_hLogicDevs.hSingleToneDetector[i],       
                  -1); 
    }

  // initialize Single Tone Detector logical devices
  {
    OTX_RESULT result;
    OTX_INT32  i;
 
    OTX_UINT32 nThreshold;
    i = 0;
      
    // get current thresholds
    result = OtxDrvGetAttributeValueUint32(
                g_hLogicDevs.hSingleToneDetector[i], 
                OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_TONE_ENERGY, 
                &nThreshold);      
    result = OtxDrvGetAttributeValueUint32(
                g_hLogicDevs.hSingleToneDetector[i], 
                OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_VARIANCE_ENERGY, 
                &nThreshold);       
    result = OtxDrvGetAttributeValueUint32(
                g_hLogicDevs.hSingleToneDetector[i], 
                OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_PAUSE_ENERGY, 
                &nThreshold); 

    // Modifing default thresholds
    nThreshold = 1000;
    // set OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_TONE_ENERGY  
    result = OtxDrvSetAttributeValueUint32(
                g_hLogicDevs.hSingleToneDetector[i], 
                OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_TONE_ENERGY, 
                nThreshold); 
      
    nThreshold = 200;
    // set OTX_ATTR_SINGLE_VARIANCE_DETECTOR_THRESHOLD_TONE_ENERGY
    result = OtxDrvSetAttributeValueUint32(
                g_hLogicDevs.hSingleToneDetector[i], 
                OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_VARIANCE_ENERGY, 
                nThreshold); 
      
    nThreshold = 200;
    // set OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_PAUSE_ENERGY
    result = OtxDrvSetAttributeValueUint32(
                g_hLogicDevs.hSingleToneDetector[i], 
                OTX_ATTR_SINGLE_TONE_DETECTOR_THRESHOLD_PAUSE_ENERGY, 
                nThreshold); 
  }
    
  for (i = 0; i < 2; i++) {
      
    OTX_UINT32    sqFreq[2];     // An array of up to 6 frequencies to be detected.
    OTX_INT32     coFreq = 2;    // Number of elements in <p sqFreq>
      
    sqFreq[0] = 440;  // Freq A in Hz
    sqFreq[1] = 620;  // Freq B in Hz       
            
    // initialize the Generic Single Tone Detector. Use a couple of DTMF tone frequencies as an example
    result =  OtxToneSingleDetectorInit(
                 g_hLogicDevs.hSingleToneDetector[i],
                 OTX_DATA_FORMAT_COMPAND8_A_LAW,   
                 sqFreq,          
                 coFreq,          
                 OTX_TRUE,        
                 OTX_TRUE,        
                 250);
      
    // enable logical device
    result = OtxDrvEnable(
                g_hLogicDevs.hSingleToneDetector[i],  
                OTX_TASK_SYNC, 
                5000L); 
  }    

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SINGLE_DETECTOR>
    <nl><t OtxDataFormatE>
    <nl><f OtxToneSingleDetectorStatus>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneSingleDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Monitored Data Stream
  IN OTX_UINT32     sqFreq[],         // @parm An array of up to 6 frequencies to be detected.
  IN OTX_INT32      coFreq,           // @parm Number of elements in <p sqFreq>
  IN OTX_BOOL       bStoreInQueue,    // @parm If OTX_TRUE, generate events and store the received digits
  IN OTX_BOOL       bReportPause,     // @parm If OTX_TRUE the detector generates an event when a tone ends
  IN OTX_INT16      nMinDuration      // @parm Minimum duration required to validate a digit
);


/*******************************************************************************

  @func
    This function allows the user to poll and check for the most recently
    received single tone. If a tone was received, the function returns 
    OTX_S_OK and the last code is in the cCode parameter. If no tone was detected, 
    the function returns OTX_S_FALSE.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_SINGLE_DETECTOR>
    <nl><f OtxToneSingleDetectorInit>
  
  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneSingleDetectorStatus(
  IN  OTX_HANDLE    hDevice,          // @parm Handle to the Logical device
  OUT char          *nCode            // @parm Most recent single tone code detected (or -1 if none)
);


/*******************************************************************************

  @func
    Initialize the Logical FSK Detector Device

  @comm
    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |
  
    // create FSK Detector
    result = OtxDrvCreateLogicalDevice(
                g_hPhysDevs.hDsp, 
                OTX_LDEVICE_TONE_FSK_DETECTOR, 
                g_hEventQueue, 
                0, 
                &g_hLogicDevs.mFskDetector.hHandle); 

    // pipe connection between DSP and CODEC, see example on
    @ex <f OtxDrvConnectPipe> |


    // connect logical device
    {
      OTX_HANDLE hPipe;

      result = OtxDrvCreatePipe(
                  64, 
                  &hPipe); 

      memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
      sqTimeSlotMask[6] = 0xFF;
      result = OtxDrvConnectPipe(
                  hPipe, 
                  OTX_PIPE_INPUT,  
                  g_hPhysDevs.hLi[0],  
                  -1, 
                  sqTimeSlotMask, 
                  32); 
      result = OtxDrvConnectPipe(
                  hPipe, 
                  OTX_PIPE_OUTPUT, 
                  g_hPhysDevs.hDsp,  
                  -1, 
                  sqTimeSlotMask, 
                  32); 
        
      // connect the Fsk detector as the sink of this connection
      result = OtxDrvConnectLogicalDevice(
                  hPipe, 
                  -1, 
                  g_hLogicDevs.mFskDetector.hHandle, 
                  -1); 
    }

    // Initialize FSK Detector
    result = OtxToneFskDetectorInit(
                g_hLogicDevs.mFskDetector.hHandle, 
                OTX_DATA_FORMAT_COMPAND8_A_LAW); 

    // enable FSK Detector (always on)
    result = OtxDrvEnable(
                g_hLogicDevs.mFskDetector.hHandle,  
                OTX_TASK_SYNC, 
                5000L); 

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_TONE_FSK_DETECTOR>
    <nl><t OtxDataFormatE>
    <nl><f OtxToneSilenceDetectorInit>
    <nl><f OtxToneDtmfDetectorInit>
    <nl><f OtxToneDialtoneDetectorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneFskDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat       // @parm Data Format of the Monitored Data Stream
);


/*******************************************************************************

  @func
    Reads one value (a byte or a word depending on the data format)
    from a specified Channel

  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxToneReadVal(
  IN  OTX_HANDLE    hDevice,          // @parm Handle to the Logical device
  OUT OTX_UINT16    *nVal             // @parm Value received on a Pipe
);



#ifdef __cplusplus
}  // extern "C"
#endif

/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998 - 2003


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif  // OTX_TONE_H
